import {Component, OnDestroy, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from "@angular/router";
import {AuditLibraryService} from "./audit-library.service";
import {AuditLibraryConfigService} from "./config/config.service";
import {Audit, FilterAudit, IEventType, ObjectType} from './model/audit.model';
import {ComponentState} from "./shared/model/component-state.model";
import {ITEMS_PER_PAGE, ITEMS_PER_PAGE10, TOTAL_COUNT_RESPONSE_HEADER} from './shared/constants/pagination.constants';
import {InputDate} from './config/dayjs';
import {ASC, DEFAULT_SORT_DATA, DESC, PAGE, SORT} from "./shared/constants/navigation.constants";
import {HttpHeaders} from '@angular/common/http';
import {QueryBuilder} from './shared/query/query.builder';
import {combineLatest, finalize, Subject, takeUntil} from "rxjs";
import {AuditServiceName} from "./model/audit-service-name.enum";
import {IUser} from "./shared/model/user.model";
import {GlobalConstants} from './config/global.constants';
import {FormsModule} from "@angular/forms";
import SortDirective from "./shared/sort/sort.directive";
import FormatStandartDateTimePipe from "./shared/date/format-standart-datetime.pipe";
import {NgClass, NgIf} from "@angular/common";
import {PrettyJsonDirective} from "./shared/pretty-json/pretty-json.directive";
import {NgbInputDatepicker, NgbPagination, NgbTooltip} from "@ng-bootstrap/ng-bootstrap";
import TranslateDirective from "./shared/language/translate.directive";
import {TranslateModule} from "@ngx-translate/core";
import SortByDirective from "./shared/sort/sort-by.directive";
import Translate3Directive from "./shared/language/translate3.directive";
import {FaIconComponent} from "@fortawesome/angular-fontawesome";
import {TranslationModule} from "./shared/language/translation.module";
import {NoDataComponent} from "./shared/no-data/no-data.component";
import {ItemsPerPageComponent} from "./shared/items-per-page/items-per-page.component";
import {AuditServiceNameComponent} from "./shared/audit-service-name/audit-service-name.component";
import {AuditEventTypeComponent} from "./shared/audit-event-type/event-type.component";
import {AuditObjectTypeComponent} from "./shared/audit-object-type/object-type.component";
import {AuditUsersComponent} from "./shared/audit-users/users.component";
import {LoadingDataComponent} from "./shared/loading-data/loading-data.component";

@Component({
    selector: 'lib-audit-library',
    standalone: true,
    imports: [
        FormsModule,
        SortDirective,
        FormatStandartDateTimePipe,
        NgClass,
        PrettyJsonDirective,
        NgbTooltip,
        NgbPagination,
        NgIf,
        TranslateDirective,
        NgbInputDatepicker,
        TranslateModule,
        SortByDirective,
        Translate3Directive,
        FaIconComponent,
        TranslationModule,
        NoDataComponent,
        ItemsPerPageComponent,
        AuditServiceNameComponent,
        AuditEventTypeComponent,
        AuditObjectTypeComponent,
        AuditUsersComponent,
        LoadingDataComponent
    ],
    templateUrl: 'audit-library.component.html',
    styleUrls: ['audit-library.component.scss'],
})
export class AuditLibraryComponent implements OnInit, OnDestroy {
    audits: Audit[] | null = null;

    // Pagination data
    itemsPerPage = ITEMS_PER_PAGE;
    page?: number;
    predicate!: string;
    ascending!: boolean;
    totalItems = 0;
    ngbPaginationPage = 1;

    // Filter data
    customFilter = new FilterAudit();
    animationState = 'out';

    componentState = new ComponentState();
    resetEventType = false;

    destroy$ = new Subject<void>();

    protected readonly InputDate = InputDate;
    protected readonly ITEMS_PER_PAGE10 = ITEMS_PER_PAGE10;

    constructor(private auditService: AuditLibraryService,
                private activatedRoute: ActivatedRoute,
                private configService: AuditLibraryConfigService,
                private router: Router) {
        configService.configureLibrary();
    }

    ngOnInit(): void {
        const data = this.activatedRoute.snapshot.data;
        const endpoint = data['endpoint'];
        const microservice = data['microservice'];

        if (endpoint && microservice || endpoint) {
            this.configService.setConfig(endpoint, microservice ?? undefined);
            this.handleNavigation();
        }
    }

    ngOnDestroy(): void {
        this.destroy$.next();
        this.destroy$.complete();
    }

    loadData(page: number): void {
        this.componentState.isLoading = true;

        const load = this.setupLoadObject(page - 1);

        this.auditService
            .query(load)
            .pipe(
                takeUntil(this.destroy$),
                finalize(() => (this.componentState.isLoading = false)),
            )
            .subscribe({
                next: res => this.onSuccess(res.body, res.headers, page),
            });
    }

    loadPage(page?: number): void {
        const pageToLoad: number = page ?? this.page ?? 1;
        this.loadData(pageToLoad);
    }

    transition(): void {
        this.router.navigate([], {
            relativeTo: this.activatedRoute,
            queryParams: {
                page: this.page,
                sort: `${this.predicate},${this.ascending ? ASC : DESC}`,
            },
        });
    }

    toggleSearchVisibility() {
        this.componentState.showDiw = !this.componentState.showDiw;
        if (this.componentState.showDiw) {
            this.animationState = this.animationState === 'out' ? 'in' : 'out';
        }
    }

    reset(): void {
        this.componentState.isReset = !this.componentState.isReset;
        this.componentState.isLoading = false;
        this.componentState.searchValue = '';
        this.componentState.isFiltered = false;

        this.customFilter = new FilterAudit();
        this.loadPage(1);
    }

    onCollapseMenu(index: number): void {
        this.componentState.isCollapsed[index] = !this.componentState.isCollapsed[index];
    }

    onObjectTypeClick(objectCode: string | undefined, objectId: string): void {
        if (objectId) {
            if (objectCode && !objectCode.includes('DELETE')) {
                this.router.navigateByUrl(objectId);
            }
        }
    }

    trackIdentity(_index: number, item: Audit): string {
        return item.objectId!;
    }

    protected onDateInput($event: any, dateType: string): void {
        const parsedDate = GlobalConstants.parseDateString($event.target.value);
        if (parsedDate) {
            if (dateType.includes(InputDate.ACCESS_DATE_FROM)) {
                this.customFilter.eventDateStart = parsedDate;
            } else if (dateType.includes(InputDate.ACCESS_DATE_TO)) {
                this.customFilter.eventDateEnd = parsedDate;
            }
        }
    }

    protected onSelectedEventTypeEmitter($event: { eventType: IEventType }): void {
        this.customFilter.auditEventType = $event.eventType;
    }

    protected onSelectedObjectTypeEmitter($event: { objectType: ObjectType }): void {
        this.customFilter.objectType = $event.objectType;
        if (!this.customFilter.auditEventType?.objectType?.includes($event.objectType)) {
            this.customFilter.auditEventType = undefined;
            this.resetEventType = !this.resetEventType;
        }
    }

    protected onSelectedUserEmitter($event: { user: IUser }): void {
        this.customFilter.user = $event.user;
    }

    protected onSelectedServiceName($event: { item: keyof typeof AuditServiceName | null }) {
        this.customFilter.serviceName = $event.item;
    }

    protected onChangePageSize($event: { item: number | null }): void {
        this.itemsPerPage = $event.item ?? ITEMS_PER_PAGE10;
        this.loadPage(1);
    }

    private handleNavigation(): void {
        combineLatest([this.activatedRoute.data, this.activatedRoute.queryParamMap]).subscribe(([data, params]) => {
            const page = params.get(PAGE);
            const pageNumber = +(page ?? 1);
            const sort = (params.get(SORT) ?? data[DEFAULT_SORT_DATA]).split(',');
            const predicate = sort[0];
            const ascending = sort[1] === ASC;

            if (pageNumber !== this.page || predicate !== this.predicate || ascending !== this.ascending) {
                this.predicate = predicate;
                this.ascending = ascending;
                this.loadPage(pageNumber);
            }
        });
    }

    private sort(): string[] {
        return [`${this.predicate},${this.ascending ? ASC : DESC}`];
    }

    private onSuccess(audits: Audit[] | null, headers: HttpHeaders, page: number): void {
        this.page = page;
        this.ngbPaginationPage = this.page;
        this.totalItems = Number(headers.get(TOTAL_COUNT_RESPONSE_HEADER));

        this.componentState.isCollapsed = [];
        this.audits = audits ?? [];
        this.transition();
    }

    private setupLoadObject(page: number): any {
        const load = {
            page,
            size: this.itemsPerPage,
            sort: this.sort(),
        };
        Object.assign(load, QueryBuilder.auditBuilder(this.customFilter));
        return load;
    }

}
