import {Injectable} from '@angular/core';
import {AuditLibraryConfigService} from "./config/config.service";
import {map, Observable} from 'rxjs';
import {Audit, AuditEventTypes, ObjectType} from "./model/audit.model";
import {HttpClient, HttpResponse} from '@angular/common/http';
import {createRequestOption} from './shared/request/request-util';
import {ApplicationConfigService} from './config/application-config.service';
import {IUser} from "./shared/model/user.model";

export type EntityResponseType = HttpResponse<Audit>;
export type EntityArrayResponseType = HttpResponse<Audit[]>;

@Injectable({
    providedIn: 'root'
})
export class AuditLibraryService {

    constructor(private configService: AuditLibraryConfigService,
                private http: HttpClient,
                private applicationConfigService: ApplicationConfigService,
    ) {
    }

    query(req?: any): Observable<EntityArrayResponseType> {
        const options = createRequestOption(req);
        return this.http
            .get<Audit[]>(this.applicationConfigService.getEndpointFor(this.configService.endpointUrl ?? '', this.configService.microservice ?? undefined), {
                params: options,
                observe: 'response'
            })
            .pipe(map(res => this.convertDetailData(res)));
    }

    getDataLogsByObjectType(objectType: ObjectType): Observable<HttpResponse<AuditEventTypes[]>> {
        return this.http.post<AuditEventTypes[]>(this.applicationConfigService.getEndpointFor(`${this.configService.endpointUrl?.slice(0, -1) ?? ''}-event-types/by-object-type`, this.configService.microservice ?? undefined), objectType, {observe: 'response'});
    }

    searchDataLogs(req?: any): Observable<HttpResponse<AuditEventTypes[]>> {
        const options = createRequestOption(req);

        return this.http.get<AuditEventTypes[]>(
            this.applicationConfigService.getEndpointFor(`${this.configService.endpointUrl?.slice(0, -1) ?? ''}-event-types/search`, this.configService.microservice ?? undefined), {
                params: options,
                observe: 'response',
            });
    }

    queryObjectType(): Observable<any> {
        return this.http.get<ObjectType[]>(
            this.applicationConfigService.getEndpointFor(`${this.configService.endpointUrl?.slice(0, -1) ?? ''}-event-types/audits-objects`, this.configService.microservice ?? undefined), {
                observe: 'response',
            });
    }

    queryUsers(req?: any): Observable<any> {
        const options = createRequestOption(req);

        return this.http.get<IUser[]>(
            this.applicationConfigService.getEndpointFor(`${this.configService.endpointUrl ?? ''}/users`, this.configService.microservice ?? undefined), {
                params: options,
                observe: 'response',
            });
    }


    private convertDetailData(res: HttpResponse<Audit[]>): HttpResponse<Audit[]> {
        return res.clone({
            body: res.body ? this.parseData(res.body) : [],
        });
    }

    private parseData(audits: Audit[]): Audit[] {
        return audits.map(audit => {
            if (audit.data) {
                const parsedData = JSON.parse(audit.data);
                audit.objectKeysData = this.convertValuesToObject(parsedData);
                audit.eventTypeData = this.convertValuesToObject(audit.eventType);
            }
            if (audit.previous) {
                const prevData = JSON.parse(audit.previous);
                audit.objectKeysPrevData = this.convertValuesToObject(prevData);
            }
            if (audit.current) {
                const currentData = JSON.parse(audit.current);
                audit.objectKeysCurrentData = this.convertValuesToObject(currentData);
            }
            return audit;
        });
    }

    private convertValuesToObject(obj: { [key: string]: any }): any {
        const convertedObj: { [key: string]: any } = {};
        // Loop through each key-value pair in the input object
        for (const key in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, key)) {
                const value = obj[key];
                // Check if the value is a string
                if (typeof value === 'string') {
                    // Find the position of the first '{' and the last '}' in the string
                    const startIndex = value.indexOf('{');
                    const endIndex = value.lastIndexOf('}');
                    if (startIndex !== -1 && endIndex !== -1) {
                        try {
                            // Extract the nested JSON object from the string
                            const nestedJSONString = value.substring(startIndex, endIndex + 1);
                            // Parse the nested JSON object
                            const parsedJSON = JSON.parse(nestedJSONString);
                            // Store the parsed JSON object
                            convertedObj[key] = parsedJSON;
                        } catch (error) {
                            // If parsing fails, store the original value
                            convertedObj[key] = value;
                        }
                    } else {
                        // If nested object braces are not found, store the original value
                        convertedObj[key] = value;
                    }
                } else {
                    // If the value is not a string, store it as is
                    convertedObj[key] = value;
                }
            }
        }
        return convertedObj;
    }
}
