import dayjs from 'dayjs/esm';
import { isDayjs } from 'dayjs';
import { AbstractControl, ValidatorFn } from '@angular/forms';
import {DATE_FORMAT, MEDIUM_DATE_FORMAT } from '../shared/constants/input.constants';

export const SESSION_KEY = 'SessionKey';
export const AUTHENTICATION_TOKEN = 'authenticationToken';

export class GlobalConstants {
  public static maxDate = { year: new Date().getFullYear(), month: new Date().getMonth() + 1, day: new Date().getDate() };

  public static isMinDateGreaterThanMaxDate(minDate?: dayjs.Dayjs, maxDate?: dayjs.Dayjs): boolean {
    if (!maxDate || !minDate) {
      return false;
    }

    return minDate.isAfter(maxDate);
  }

  public static areValuesEqual(obj1: any, obj2: any): boolean {
    return JSON.stringify(obj1) === JSON.stringify(obj2);
  }

  public static isDifferent = (value: string, current: string, existing: string | null | undefined): boolean =>
    value !== current && value !== existing;

  public static parseDateString(inputDate: string): dayjs.Dayjs | null {
    const regexPattern = /^(\d{2})(\d{2})(\d{4})$/;
    const match = this.removeDotsFromData(inputDate).match(regexPattern);

    if (match && inputDate.length >= 8) {
      const [, day, month, year] = match;
      const formattedDate = `${day}.${month}.${year}`;
      return dayjs(formattedDate, MEDIUM_DATE_FORMAT);
    }

    return null;
  }

  public static formatDateToDateForm(date: any): string | null {
    if (isDayjs(date)) {
      return date.format(DATE_FORMAT);
    } else if (date) {
      return dayjs(date).format(DATE_FORMAT);
    }
    return null;
  }

  public static removeDotsFromData(inputDate: string): string {
    return inputDate.replace(/\./g, '');
  }

  public static allowedNumbersValidator(allowedValues: number[]): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
      const value = control.value;

      if (value && !allowedValues.includes(value)) {
        return { allowedNumbers: true };
      }

      return null;
    };
  }

  public static getEnumKeyByEnumValue(enumType: any, enumValue: string): string | undefined {
    const keys = Object.keys(enumType).filter(key => enumType[key] === enumValue);
    return keys.length > 0 ? keys[0] : undefined;
  }

  public static convertToUnderscoreCase(input: string): string {
    const result = input.replace(/[A-Z]/g, match => '_' + match.toLowerCase());
    if (result.startsWith('_')) {
      return result.slice(1);
    }
    return result;
  }

  public static adjustMaxWidth(tableId: string, width: number): void {
    const table = document.getElementById(`${tableId}`);
    if (table) {
      table.style.maxWidth = width === 250 ? 'calc(100vw - 305px)' : 'calc(100vw - 115px)';
    }
  }

  public static adjustTableModalMaxWidth(tableId: string, modalWidth: number): void {
    modalWidth -= 65;
    const table = document.getElementById(`${tableId}`);
    if (table) {
      table.style.maxWidth = `calc(${modalWidth}px)`;
    }
  }

  public static removeErrors(keys: string[] | undefined, control: AbstractControl | null): void {
    if (!control || !keys || keys.length === 0) {
      return;
    }

    const remainingErrors = keys.reduce(
      (errors, key) => {
        delete errors[key];
        return errors;
      },
      { ...control.errors },
    );

    control.setErrors(remainingErrors);

    if (Object.keys(control.errors || {}).length === 0) {
      control.setErrors(null);
    }
  }

  public static addErrors(errors: { [key: string]: any } | undefined, control: AbstractControl | null): void {
    if (!control || !errors) {
      return;
    }

    control.setErrors({ ...control.errors, ...errors });
  }

  // public static getFileExtension(fileFormat: keyof typeof ReportFormatType): string {
  //   switch (fileFormat) {
  //     case ReportFormatType.PDF:
  //       return 'pdf';
  //     case ReportFormatType.CSV:
  //       return 'csv';
  //     default:
  //       return '';
  //   }
  // }
}
